use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
Adams
Alders
Andy
AnnoCPAN
AttributeRegistry
AutoUse
Bell
Beresford
Bowers
Boyd
CLI
CPAN
Christopher
Chuck
Class
Config
Copypasta
Corneliu
Curtis
DEPRECATIONS
DROLSKY
DROLSKY's
Daignault
Dave
Denny
Deprecated
Desmond
Doug
Etheridge
Executor
Extensibility
Gregory
Harald
HasTimeReport
Haye
Heady
Humphrey
Instance
Jack
Jeremy
Jonathan
Jörg
Karen
Krieg
Larry
Layne
Leszczynski
Load
MetaCPAN
Method
Moose
Neil
OO
Oji
Olaf
Oschwald
Otsuka
Ovid
Parallel
ParameterizedInstances
Paul
PayPal
Petrea
Poe
Refactor
Report
Reporting
Role
Rolsky
Rolsky's
Runner
STDOUT
Sequential
Stefan
Steven
Stowe
Stuckdownawell
TCM
TODO
Test
Tim
Time
Tom
Tutorial
Udo
Util
Velti
Vroom
Williams
arisdottle
autarch
catchgithub
charles_adams
clayne
de
denny
dirs
distro
djgoku
drolsky
ether
github
goschwald
hardcode
hashrefs
invocant
jjrs
jns
jrubinator
kwakwaversal
la
larryl
lib
madcityzen
maflick88
mark
mephinet
metadata
munge
munges
namespace
nawglan
neil
ok
olaf
online
ovid
parallelizable
parallelize
parallelized
parallelizes
parameterized
pboyd
prereq
rebless
runtime
startup
stefan
strawman
stuckdownawell
subtest
subtests
tcm
teardown
tom
toolkits
un
vroom
xUnit
